from typing import Any, Dict, Optional

from openai import AsyncOpenAI

from esperanto.base.types import SpeechToTextModel


class OpenAISpeechToTextModel(SpeechToTextModel):
    """OpenAI speech-to-text model implementation (Whisper)."""

    def __init__(
        self,
        model_name: str,
        config: Optional[Dict[str, Any]] = None,
    ):
        super().__init__(model_name, config)
        self._client = None
        self.language = self.config.get("language")
        self.prompt = self.config.get("prompt")
        self.response_format = self.config.get("response_format", "text")
        self.temperature = self.config.get("temperature", 0)
        self.validate_config()

    @property
    def client(self) -> AsyncOpenAI:
        """Get the OpenAI client."""
        if self._client is None:
            self._client = AsyncOpenAI()
        return self._client

    @client.setter
    def client(self, value: AsyncOpenAI) -> None:
        """Set the OpenAI client."""
        self._client = value

    @client.deleter
    def client(self) -> None:
        """Delete the OpenAI client."""
        self._client = None

    @property
    def provider(self) -> str:
        return "openai"

    def validate_config(self) -> None:
        """Validate OpenAI configuration."""
        if not self.model_name:
            raise ValueError(
                "model_name must be specified for OpenAI speech-to-text model"
            )
        if self.response_format not in ["json", "text", "srt", "verbose_json", "vtt"]:
            raise ValueError(
                "response_format must be one of: json, text, srt, verbose_json, vtt"
            )

    async def transcribe(self, audio_file: str, **kwargs) -> str:
        """Transcribe audio to text using OpenAI Whisper."""
        with open(audio_file, "rb") as f:
            response = await self.client.audio.transcriptions.create(
                model=self.model_name,
                file=f,
                language=self.language,
                prompt=self.prompt,
                response_format=self.response_format,
                temperature=self.temperature,
                **kwargs,
            )

        return response



"""Embedding model implementations."""

from esperanto.providers.embedding.gemini import GeminiEmbeddingModel
from esperanto.providers.embedding.ollama import OllamaEmbeddingModel
from esperanto.providers.embedding.openai import OpenAIEmbeddingModel
from esperanto.providers.embedding.vertex import VertexEmbeddingModel

__all__ = [
    "GeminiEmbeddingModel",
    "OllamaEmbeddingModel",
    "OpenAIEmbeddingModel",
    "VertexEmbeddingModel",
]


from abc import ABC, abstractmethod
from typing import Any, Dict, Optional


class BaseModel(ABC):
    """Base class for all AI models."""

    def __init__(self, model_name: str, config: Optional[Dict[str, Any]] = {}):
        """Initialize the base model.

        Args:
            model_name: The name of the model to use
            config: Optional configuration dictionary for the model
        """
        self.model_name = model_name
        self.config = config or {}

    @abstractmethod
    def validate_config(self) -> None:
        """Validate the model configuration."""
        pass

    @property
    @abstractmethod
    def provider(self) -> str:
        """Return the provider name for this model."""
        pass



"""Factory module for creating AI service instances."""

import importlib
from typing import Any, Dict, Optional, Type

from esperanto.base.types import LanguageModel, SpeechToTextModel, TextToSpeechModel


class AIFactory:
    """Factory class for creating AI service instances."""

    # Provider module mappings
    _provider_modules = {
        "llm": {
            "openai": "esperanto.providers.llm.openai:OpenAILanguageModel",
            "anthropic": "esperanto.providers.llm.anthropic:AnthropicLanguageModel",
            "gemini": "esperanto.providers.llm.gemini:GeminiLanguageModel",
            "groq": "esperanto.providers.llm.groq:GroqLanguageModel",
            "litellm": "esperanto.providers.llm.litellm:LiteLLMLanguageModel",
            "ollama": "esperanto.providers.llm.ollama:OllamaLanguageModel",
            "openrouter": "esperanto.providers.llm.openrouter:OpenRouterLanguageModel",
            "vertex": "esperanto.providers.llm.vertex:VertexAILanguageModel",
            "vertex_anthropic": "esperanto.providers.llm.vertex:VertexAnthropicLanguageModel",
            "xai": "esperanto.providers.llm.xai:XAILanguageModel",
        },
        "stt": {
            "openai": "esperanto.providers.speech_to_text.openai:OpenAISpeechToTextModel",
            "groq": "esperanto.providers.speech_to_text.groq:GroqSpeechToTextModel",
        },
        "tts": {
            "openai": "esperanto.providers.text_to_speech.openai:OpenAITextToSpeechModel",
            "elevenlabs": "esperanto.providers.text_to_speech.elevenlabs:ElevenLabsTextToSpeechModel",
            "gemini": "esperanto.providers.text_to_speech.gemini:GeminiTextToSpeechModel",
        },
    }

    @classmethod
    def _import_provider_class(cls, service_type: str, provider: str) -> Type:
        """Dynamically import provider class.

        Args:
            service_type: Type of service (llm, stt, tts)
            provider: Provider name

        Returns:
            Provider class

        Raises:
            ValueError: If provider is not supported
            ImportError: If provider module is not installed
        """
        if service_type not in cls._provider_modules:
            raise ValueError(f"Invalid service type: {service_type}")

        provider = provider.lower()
        if provider not in cls._provider_modules[service_type]:
            raise ValueError(
                f"Provider '{provider}' not supported for {service_type}. "
                f"Supported providers: {list(cls._provider_modules[service_type].keys())}"
            )

        module_path = cls._provider_modules[service_type][provider]
        module_name, class_name = module_path.split(":")

        try:
            module = importlib.import_module(module_name)
            return getattr(module, class_name)
        except ImportError as e:
            # Get the provider package name from the module path
            provider_package = module_name.split(".")[3]  # e.g., openai, anthropic, etc.
            raise ImportError(
                f"Provider '{provider}' requires additional dependencies. "
                f"Install them with: poetry install --with {provider_package}"
            ) from e

    @classmethod
    def create_llm(
        cls, provider: str, model_name: str, config: Optional[Dict[str, Any]] = None
    ) -> LanguageModel:
        """Create a language model instance.

        Args:
            provider: Provider name
            model_name: Name of the model to use
            config: Optional configuration for the model

        Returns:
            Language model instance
        """
        provider_class = cls._import_provider_class("llm", provider)
        return provider_class(model_name=model_name, config=config or {})

    @classmethod
    def create_stt(
        cls, provider: str, model_name: str, config: Optional[Dict[str, Any]] = None
    ) -> SpeechToTextModel:
        """Create a speech-to-text model instance.

        Args:
            provider: Provider name
            model_name: Name of the model to use
            config: Optional configuration for the model

        Returns:
            Speech-to-text model instance
        """
        provider_class = cls._import_provider_class("stt", provider)
        return provider_class(model_name=model_name, config=config or {})

    @classmethod
    def create_tts(
        cls, provider: str, config: Optional[Dict[str, Any]] = None
    ) -> TextToSpeechModel:
        """Create a text-to-speech model instance.

        Args:
            provider: Provider name
            config: Optional configuration for the model

        Returns:
            Text-to-speech model instance
        """
        provider_class = cls._import_provider_class("tts", provider)
        return provider_class(config=config or {})



"""ElevenLabs text-to-speech provider."""

from typing import Any, Optional

from elevenlabs import AsyncElevenLabs
from pydantic import SecretStr

from esperanto.base import TextToSpeechModel


class ElevenLabsTextToSpeechModel(TextToSpeechModel):
    """ElevenLabs text-to-speech model."""

    def __init__(
        self,
        model_name: Optional[str] = None,
        config: Optional[dict[str, Any]] = None,
    ) -> None:
        """Initialize ElevenLabs text-to-speech model.

        Args:
            model_name: Model name
            config: Configuration dictionary
        """
        super().__init__(model_name=model_name, config=config)
        self._client: Optional[AsyncElevenLabs] = None
        self._model = self.model_name  # Set _model from model_name
        self._api_key = None  # Initialize _api_key as None
        self._voice = None  # Initialize _voice as None
        
        # Set API key from config if provided
        if config and "api_key" in config:
            self._api_key = SecretStr(config["api_key"])
        else:
            # Try to get from environment variable
            import os
            api_key = os.getenv("ELEVENLABS_API_KEY")
            if api_key:
                self._api_key = SecretStr(api_key)
                
        # Set voice from config if provided
        if config and "voice" in config:
            self._voice = config["voice"]
        else:
            # Set default voice if not provided
            self._voice = "Adam"  # Using a default voice from ElevenLabs

    @property
    def model(self) -> str:
        """Get model name.

        Returns:
            Model name
        """
        return self._model

    @property
    def provider(self) -> str:
        """Get provider name.

        Returns:
            Provider name
        """
        return "elevenlabs"

    def validate_config(self) -> None:
        """Validate configuration.

        Raises:
            ValueError: If configuration is invalid
        """
        # Validate model name
        if not self._model:
            raise ValueError("model_name must be specified")

        # Validate API key
        if not self._api_key:
            raise ValueError("api_key must be specified")

        # Validate voice
        if not self._voice:
            raise ValueError("voice must be specified")

        # Validate stability (if provided)
        stability = self.config.get("stability")
        if stability is not None:
            if (
                not isinstance(stability, (int, float))
                or stability < 0
                or stability > 1
            ):
                raise ValueError("Stability must be between 0 and 1")

        # Validate similarity boost (if provided)
        similarity_boost = self.config.get("similarity_boost")
        if similarity_boost is not None:
            if (
                not isinstance(similarity_boost, (int, float))
                or similarity_boost < 0
                or similarity_boost > 1
            ):
                raise ValueError("Similarity boost must be between 0 and 1")

        # Validate style (if provided)
        style = self.config.get("style")
        if style is not None:
            if not isinstance(style, (int, float)) or style < 0 or style > 1:
                raise ValueError("Style must be between 0 and 1")

    @property
    def client(self) -> AsyncElevenLabs:
        """Get ElevenLabs client.

        Returns:
            ElevenLabs client

        Raises:
            ValueError: If API key is not set
        """
        if self._client is None:
            if not self._api_key:
                raise ValueError("API key not set")
            self._client = AsyncElevenLabs(api_key=self._api_key.get_secret_value())
        return self._client

    @client.setter
    def client(self, client: AsyncElevenLabs) -> None:
        """Set ElevenLabs client.

        Args:
            client: ElevenLabs client
        """
        self._client = client

    @client.deleter
    def client(self) -> None:
        """Delete ElevenLabs client."""
        self._client = None

    @property
    def voice(self) -> str:
        """Get voice.

        Returns:
            Voice
        """
        return self._voice

    async def synthesize(self, text: str, output_file: str, **kwargs: Any) -> str:
        """Synthesize text to speech using ElevenLabs API.

        Args:
            text: Text to synthesize
            output_file: Path to save the audio file
            **kwargs: Additional arguments to pass to the API

        Returns:
            Path to the output audio file

        Raises:
            ValueError: If configuration is invalid or synthesis fails
        """
        try:
            self.validate_config()

            # Get optional parameters from config or kwargs
            stability = kwargs.pop("stability", self.config.get("stability"))
            similarity_boost = kwargs.pop(
                "similarity_boost", self.config.get("similarity_boost")
            )
            style = kwargs.pop("style", self.config.get("style"))

            # Create voice settings dictionary
            voice_settings = {}
            if stability is not None:
                voice_settings["stability"] = stability
            if similarity_boost is not None:
                voice_settings["similarity_boost"] = similarity_boost
            if style is not None:
                voice_settings["style"] = style

            # Create synthesis request
            audio_stream = await self.client.generate(
                text=text,
                voice=self.voice,
                model=self.model,
                voice_settings=voice_settings if voice_settings else None,
                **kwargs,  # Pass remaining kwargs
            )

            # Save to file
            with open(output_file, "wb") as f:
                async for chunk in audio_stream:
                    f.write(chunk)

            return output_file

        except Exception as e:
            raise ValueError(f"Failed to synthesize speech: {str(e)}") from e



"""OpenAI Text-to-Speech model implementation."""

import os
from typing import Any, ClassVar, Dict, Optional

import openai
from pydantic import SecretStr

from esperanto.base.types import TextToSpeechModel


class OpenAITextToSpeechModel(TextToSpeechModel):
    """OpenAI Text-to-Speech model implementation."""

    DEFAULT_MODEL: ClassVar[str] = "tts-1"
    VALID_MODELS: ClassVar[tuple] = ("tts-1", "tts-1-hd")
    VALID_VOICES: ClassVar[tuple] = ("alloy", "echo", "fable", "onyx", "nova", "shimmer")
    DEFAULT_VOICE: ClassVar[str] = "alloy"
    VALID_FORMATS: ClassVar[tuple] = ("mp3", "opus", "aac", "flac")
    DEFAULT_FORMAT: ClassVar[str] = "mp3"

    def __init__(
        self,
        model_name: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        """Initialize OpenAI Text-to-Speech model.
        
        Args:
            model_name: Model name (default: tts-1)
            config: Configuration dictionary. Can include:
                - api_key: OpenAI API key
                - model: Model name (default: tts-1)
                - voice: Voice ID (default: alloy)
                - response_format: Audio format (default: mp3)
                - speed: Speech speed (default: 1.0)
        """
        config = config or {}
        if model_name:
            config["model"] = model_name
        super().__init__(config)
        self._client = None
        
        # API configuration
        api_key = self.config.get("api_key")
        if not api_key and "OPENAI_API_KEY" in os.environ:
            api_key = os.environ["OPENAI_API_KEY"]
        self._api_key = SecretStr(api_key) if api_key else None
        
        # Model configuration
        self._model = model_name or config.get("model") or None
        self._voice = config.get("voice", self.DEFAULT_VOICE)
        self._response_format = config.get("response_format", self.DEFAULT_FORMAT)
        self._speed = float(config.get("speed", 1.0))

    @property
    def client(self) -> openai.Client:
        """Get OpenAI client."""
        if self._client is None:
            if not self.api_key:
                raise ValueError("OpenAI API key must be provided in config or OPENAI_API_KEY environment variable")
            self._client = openai.Client(api_key=self.api_key.get_secret_value())
        return self._client

    @client.setter
    def client(self, value: openai.Client) -> None:
        """Set OpenAI client."""
        self._client = value

    @property
    def api_key(self) -> Optional[SecretStr]:
        """Get API key."""
        if self._api_key:
            return self._api_key
        if openai.api_key:
            return SecretStr(openai.api_key)
        if "OPENAI_API_KEY" in os.environ:
            return SecretStr(os.environ["OPENAI_API_KEY"])
        return None

    @property
    def model(self) -> str:
        """Get model name."""
        return self._model

    @property
    def voice(self) -> str:
        """Get voice ID."""
        return self._voice

    @property
    def response_format(self) -> str:
        """Get response format."""
        return self._response_format

    @property
    def speed(self) -> float:
        """Get speech speed."""
        return self._speed

    @property
    def provider(self) -> str:
        """Get provider name."""
        return "openai"

    def validate_config(self) -> None:
        """Validate configuration."""
        # Check API key
        if not self.api_key:
            raise ValueError("api_key must be specified")
        
        # Check model name
        if not self._model:
            raise ValueError("model_name must be specified")
        if self._model not in self.VALID_MODELS:
            raise ValueError(f"Invalid model. Must be one of: {', '.join(self.VALID_MODELS)}")
        
        # Check voice
        if not self._voice:
            raise ValueError("voice must be specified")
        if self._voice not in self.VALID_VOICES:
            raise ValueError(f"Invalid voice. Must be one of: {', '.join(self.VALID_VOICES)}")
        
        # Check response format
        if not self._response_format:
            raise ValueError("response_format must be specified")
        if self._response_format not in self.VALID_FORMATS:
            raise ValueError(f"Invalid response_format. Must be one of: {', '.join(self.VALID_FORMATS)}")
        
        # Check speed
        if not isinstance(self._speed, (int, float)):
            raise ValueError("speed must be a number")
        if not 0.25 <= self._speed <= 4.0:
            raise ValueError("Speed must be between 0.25 and 4.0")

    async def synthesize(self, text: str, output_file: str, **kwargs: Any) -> str:
        """Synthesize text to speech using OpenAI's API.
        
        Args:
            text: Text to synthesize
            output_file: Path to save the audio file
            **kwargs: Additional arguments to pass to the API
            
        Returns:
            Path to the output audio file
            
        Raises:
            ValueError: If configuration is invalid or synthesis fails
        """
        try:
            self.validate_config()

            # Extract known parameters from kwargs
            voice = kwargs.pop("voice", self.voice)
            speed = kwargs.pop("speed", self.speed)

            # Create synthesis request
            response = self.client.audio.speech.create(  # OpenAI's create is synchronous
                model=self.model,
                voice=voice,
                input=text,
                response_format=self.response_format,
                speed=speed,
                **kwargs,  # Pass remaining kwargs
            )

            # Save to file
            with open(output_file, "wb") as f:
                for chunk in response.iter_bytes():  # Handle streaming response
                    f.write(chunk)

            return output_file

        except Exception as e:
            raise ValueError(f"Failed to synthesize speech: {str(e)}") from e



from typing import Any, Dict

from langchain_community.chat_models import ChatLiteLLM
from langchain_core.language_models.chat_models import BaseChatModel

from esperanto.base.types import LanguageModel


class LiteLLMLanguageModel(LanguageModel):
    """LiteLLM language model implementation."""

    def __init__(
        self,
        model_name: str,
        config: Dict[str, Any] = {},
    ):
        super().__init__(model_name, config)
        if not model_name:
            raise ValueError("model_name must be specified for LiteLLM language model")
        self.max_tokens = self.config.get("max_tokens", 850)
        self.temperature = self.config.get("temperature", 1.0)
        self.streaming = self.config.get("streaming", True)
        self.top_p = self.config.get("top_p", 0.9)
        self.api_base = self.config.get("api_base")

    @property
    def provider(self) -> str:
        return "litellm"

    def validate_config(self) -> None:
        """Validate LiteLLM configuration."""
        if not self.model_name:
            raise ValueError("model_name must be specified for LiteLLM language model")

    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model."""
        config = {
            "model": self.model_name,
            "max_tokens": self.max_tokens,
            "temperature": self.temperature,
            "streaming": self.streaming,
            "top_p": self.top_p,
        }
        if self.api_base:
            config["api_base"] = self.api_base

        return ChatLiteLLM(**config)



import os
from typing import Any, Dict, Optional

from langchain_core.language_models.chat_models import BaseChatModel
from langchain_google_vertexai import ChatVertexAI
from langchain_google_vertexai.model_garden import ChatAnthropicVertex

from esperanto.base.types import LanguageModel


class VertexAILanguageModel(LanguageModel):
    """Vertex AI language model implementation."""

    def __init__(
        self,
        model_name: str,
        config: Dict[str, Any] = {},
    ):
        if not model_name:
            raise ValueError(
                "model_name must be specified for Vertex AI language model"
            )
        super().__init__(model_name, config)
        self.max_tokens = self.config.get("max_tokens", 850)
        self.temperature = self.config.get("temperature", 1.0)
        self.streaming = self.config.get("streaming", True)
        self.top_p = self.config.get("top_p", 0.9)

        # Handle project and location
        self.project = self.config.get("project") or os.environ.get(
            "VERTEX_PROJECT", "no-project"
        )
        self.location = self.config.get("location") or os.environ.get(
            "VERTEX_LOCATION", "us-central1"
        )

    @property
    def provider(self) -> str:
        return "vertex"

    def validate_config(self) -> None:
        """Validate Vertex AI configuration."""
        if not self.model_name:
            raise ValueError(
                "model_name must be specified for Vertex AI language model"
            )
        if not self.project:
            raise ValueError(
                "project must be specified in config or VERTEX_PROJECT environment variable"
            )
        if not self.location:
            raise ValueError(
                "location must be specified in config or VERTEX_LOCATION environment variable"
            )

    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model."""
        return ChatVertexAI(
            model_name=self.model_name,
            max_output_tokens=self.max_tokens,
            temperature=self.temperature,
            top_p=self.top_p,
            project=self.project,
            location=self.location,
            streaming=self.streaming,
        )


class VertexAnthropicLanguageModel(LanguageModel):
    """Vertex AI Anthropic language model implementation."""

    def __init__(
        self,
        model_name: str,
        config: Optional[Dict[str, Any]] = None,
    ):
        if not model_name:
            raise ValueError(
                "model_name must be specified for Vertex Anthropic language model"
            )
        super().__init__(model_name, config)
        self.max_tokens = self.config.get("max_tokens", 850)
        self.temperature = self.config.get("temperature", 1.0)
        self.streaming = self.config.get("streaming", True)
        self.top_p = self.config.get("top_p", 0.9)

        # Handle project and location
        self.project = self.config.get("project") or os.environ.get(
            "VERTEX_PROJECT", "no-project"
        )
        self.location = self.config.get("location") or os.environ.get(
            "VERTEX_LOCATION", "us-central1"
        )

    @property
    def provider(self) -> str:
        return "vertex_anthropic"

    def validate_config(self) -> None:
        """Validate Vertex Anthropic configuration."""
        if not self.model_name:
            raise ValueError(
                "model_name must be specified for Vertex Anthropic language model"
            )
        if not self.project:
            raise ValueError(
                "project must be specified in config or VERTEX_PROJECT environment variable"
            )
        if not self.location:
            raise ValueError(
                "location must be specified in config or VERTEX_LOCATION environment variable"
            )

    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model."""
        return ChatAnthropicVertex(
            model_name=self.model_name,
            max_output_tokens=self.max_tokens,
            temperature=self.temperature,
            top_p=self.top_p,
            project=self.project,
            location=self.location,
            streaming=self.streaming,
        )



"""Anthropic language model implementation."""

import os
from typing import Any, ClassVar, Dict, Optional

from langchain_anthropic import ChatAnthropic
from langchain_core.language_models.chat_models import BaseChatModel
from pydantic import SecretStr

from esperanto.base.types import LanguageModel


class AnthropicLanguageModel(LanguageModel):
    """Anthropic language model implementation."""

    DEFAULT_MAX_TOKENS: ClassVar[int] = 850
    DEFAULT_TEMPERATURE: ClassVar[float] = 1.0
    DEFAULT_TOP_P: ClassVar[float] = 0.9
    MIN_TEMPERATURE: ClassVar[float] = 0.0
    MAX_TEMPERATURE: ClassVar[float] = 1.0
    MIN_TOP_P: ClassVar[float] = 0.0
    MAX_TOP_P: ClassVar[float] = 1.0

    def __init__(
        self,
        model_name: str,
        config: Dict[str, Any] = {},
    ):
        """Initialize Anthropic language model.

        Args:
            model_name: Name of the model to use. Defaults to claude-3-opus-20240229.
            config: Configuration dictionary. Can include:
                - api_key: Anthropic API key
                - base_url: Optional API base URL
                - max_tokens: Maximum number of tokens to generate (default: 850)
                - temperature: Sampling temperature (default: 1.0)
                - top_p: Nucleus sampling parameter (default: 0.9)
                - streaming: Whether to stream responses (default: True)
        """
        super().__init__(model_name=model_name, config=config)

        # Model configuration
        self._max_tokens = self.config.get("max_tokens", self.DEFAULT_MAX_TOKENS)
        self._temperature = self.config.get("temperature", self.DEFAULT_TEMPERATURE)
        self._top_p = self.config.get("top_p", self.DEFAULT_TOP_P)
        self._streaming = self.config.get("streaming", True)

        # API configuration
        api_key = self.config.get("api_key") or os.environ.get("ANTHROPIC_API_KEY", "")
        self._api_key = (
            api_key if isinstance(api_key, SecretStr) else SecretStr(api_key)
        )
        self._base_url = self.config.get("base_url") or os.environ.get(
            "ANTHROPIC_API_BASE"
        )

        self.validate_config()

    @property
    def provider(self) -> str:
        """Get provider name."""
        return "anthropic"

    @property
    def max_tokens(self) -> int:
        """Get maximum tokens setting."""
        return self._max_tokens

    @property
    def temperature(self) -> float:
        """Get temperature setting."""
        return self._temperature

    @property
    def top_p(self) -> float:
        """Get top_p setting."""
        return self._top_p

    @property
    def streaming(self) -> bool:
        """Get streaming setting."""
        return self._streaming

    @property
    def api_key(self) -> SecretStr:
        """Get API key."""
        return self._api_key

    @property
    def base_url(self) -> Optional[str]:
        """Get base URL."""
        return self._base_url

    def validate_config(self) -> None:
        """Validate Anthropic configuration.

        Raises:
            ValueError: If any configuration parameter is invalid.
        """
        if not self.model_name:
            raise ValueError(
                "model_name must be specified for Anthropic language model"
            )
        if not self.api_key.get_secret_value():
            raise ValueError(
                "api_key must be specified in config or ANTHROPIC_API_KEY environment variable"
            )
        if not isinstance(self.max_tokens, int) or self.max_tokens <= 0:
            raise ValueError("max_tokens must be a positive integer")
        if not self.MIN_TEMPERATURE <= self.temperature <= self.MAX_TEMPERATURE:
            raise ValueError(
                f"temperature must be between {self.MIN_TEMPERATURE} and {self.MAX_TEMPERATURE}"
            )
        if not self.MIN_TOP_P <= self.top_p <= self.MAX_TOP_P:
            raise ValueError(
                f"top_p must be between {self.MIN_TOP_P} and {self.MAX_TOP_P}"
            )

    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model.

        Returns:
            BaseChatModel: LangChain chat model instance.

        Raises:
            ValueError: If configuration is invalid.
        """
        try:
            return ChatAnthropic(
                model=self.model_name,
                temperature=self.temperature,
                max_tokens=self.max_tokens,
                timeout=None,
                max_retries=2,
                anthropic_api_key=self.api_key,
                streaming=self.streaming,
                top_p=self.top_p,
            )

        except Exception as e:
            raise ValueError(f"Failed to create LangChain model: {str(e)}") from e



"""Text-to-speech model implementations."""

from esperanto.providers.text_to_speech.elevenlabs import ElevenLabsTextToSpeechModel
from esperanto.providers.text_to_speech.gemini import GeminiTextToSpeechModel
from esperanto.providers.text_to_speech.openai import OpenAITextToSpeechModel

__all__ = [
    "ElevenLabsTextToSpeechModel",
    "GeminiTextToSpeechModel",
    "OpenAITextToSpeechModel",
]


"""Language model implementations."""

from esperanto.providers.llm.anthropic import AnthropicLanguageModel
from esperanto.providers.llm.gemini import GeminiLanguageModel
from esperanto.providers.llm.groq import GroqLanguageModel
from esperanto.providers.llm.litellm import LiteLLMLanguageModel
from esperanto.providers.llm.ollama import OllamaLanguageModel
from esperanto.providers.llm.openai import OpenAILanguageModel
from esperanto.providers.llm.openrouter import OpenRouterLanguageModel
from esperanto.providers.llm.vertex import (
    VertexAILanguageModel,
    VertexAnthropicLanguageModel,
)
from esperanto.providers.llm.xai import XAILanguageModel

__all__ = [
    "AnthropicLanguageModel",
    "GeminiLanguageModel",
    "GroqLanguageModel",
    "LiteLLMLanguageModel",
    "OllamaLanguageModel",
    "OpenAILanguageModel",
    "OpenRouterLanguageModel",
    "VertexAILanguageModel",
    "VertexAnthropicLanguageModel",
    "XAILanguageModel",
]


"""
Esperanto - A unified interface for various AI model providers
"""

__version__ = "0.1.0"

from .base import (
    AudioSynthesis,
    AudioTranscription,
    BaseModel,
    ChatCompletion,
    Embedding,
    EmbeddingModel,
    LanguageModel,
    Message,
    SpeechToTextModel,
    TextToSpeechModel,
)
from .providers.embedding import (
    GeminiEmbeddingModel,
    OllamaEmbeddingModel,
    OpenAIEmbeddingModel,
    VertexEmbeddingModel,
)
from .providers.llm import (
    AnthropicLanguageModel,
    GeminiLanguageModel,
    GroqLanguageModel,
    LiteLLMLanguageModel,
    OllamaLanguageModel,
    OpenAILanguageModel,
    OpenRouterLanguageModel,
    VertexAILanguageModel,
    VertexAnthropicLanguageModel,
    XAILanguageModel,
)
from .providers.speech_to_text import (
    GroqSpeechToTextModel,
    OpenAISpeechToTextModel,
)
from .providers.text_to_speech import (
    ElevenLabsTextToSpeechModel,
    GeminiTextToSpeechModel,
    OpenAITextToSpeechModel,
)

__all__ = [
    # Base classes
    "BaseModel",
    "LanguageModel",
    "EmbeddingModel",
    "SpeechToTextModel",
    "TextToSpeechModel",
    # Data classes
    "Message",
    "ChatCompletion",
    "Embedding",
    "AudioTranscription",
    "AudioSynthesis",
    # Embedding models
    "GeminiEmbeddingModel",
    "OllamaEmbeddingModel",
    "OpenAIEmbeddingModel",
    "VertexEmbeddingModel",
    # Language models
    "AnthropicLanguageModel",
    "GeminiLanguageModel",
    "GroqLanguageModel",
    "LiteLLMLanguageModel",
    "OllamaLanguageModel",
    "OpenAILanguageModel",
    "OpenRouterLanguageModel",
    "VertexAILanguageModel",
    "VertexAnthropicLanguageModel",
    "XAILanguageModel",
    # Speech-to-text models
    "OpenAISpeechToTextModel",
    "GroqSpeechToTextModel",
    # Text-to-speech models
    "ElevenLabsTextToSpeechModel",
    "GeminiTextToSpeechModel",
    "OpenAITextToSpeechModel",
]



from typing import Any, Dict, List, Optional

import google.generativeai as genai  # type: ignore

from esperanto.base.types import EmbeddingModel


class GeminiEmbeddingModel(EmbeddingModel):
    """Gemini embedding model implementation."""

    def __init__(self, model_name: str, config: Optional[Dict[str, Any]] = None):
        if not model_name:
            raise ValueError("model_name must be specified for Gemini embedding model")
        super().__init__(model_name, config)

    @property
    def provider(self) -> str:
        return "gemini"

    def validate_config(self) -> None:
        """Validate Gemini configuration."""
        pass

    async def embed(self, texts: List[str], **kwargs) -> List[List[float]]:
        """Generate embeddings for the given texts."""
        results = []
        for text in texts:
            model_name = (
                self.model_name
                if self.model_name.startswith("models/")
                else f"models/{self.model_name}"
            )
            result = genai.embed_content(model=model_name, content=text, **kwargs)
            results.append(result["embedding"])
        return results



from typing import Any, Dict, List, Optional

from openai import AsyncOpenAI

from esperanto.base.types import EmbeddingModel


class OpenAIEmbeddingModel(EmbeddingModel):
    """OpenAI embedding model implementation."""

    def __init__(self, model_name: str, config: Optional[Dict[str, Any]] = None):
        super().__init__(model_name, config)
        self.validate_config()
        self._client: Optional[AsyncOpenAI] = None

    @property
    def client(self) -> AsyncOpenAI:
        """Lazy initialization of OpenAI client."""
        if self._client is None:
            self._client = AsyncOpenAI()
        return self._client

    @client.setter
    def client(self, value: AsyncOpenAI) -> None:
        """Set the OpenAI client."""
        self._client = value

    @client.deleter
    def client(self) -> None:
        """Delete the OpenAI client."""
        self._client = None

    @property
    def provider(self) -> str:
        return "openai"

    def validate_config(self) -> None:
        """Validate OpenAI configuration."""
        if not self.model_name:
            raise ValueError("model_name must be specified for OpenAI embedding model")

    async def embed(self, texts: List[str], **kwargs) -> List[List[float]]:
        """Generate embeddings for the given texts."""
        # Preprocess texts
        texts = [text.replace("\n", " ") for text in texts]

        # Get embeddings
        response = await self.client.embeddings.create(
            input=texts, model=self.model_name, **kwargs
        )

        return [data.embedding for data in response.data]



"""Base types for the esperanto package."""

from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Dict, List, Optional

from langchain_core.language_models.chat_models import BaseChatModel

from esperanto.base.model import BaseModel


@dataclass
class Message:
    """A message in a conversation."""
    role: str
    content: str


@dataclass
class ChatCompletion:
    """A chat completion response."""
    messages: List[Message]
    model: str
    usage: Dict[str, int]
    finish_reason: Optional[str] = None


@dataclass
class Embedding:
    """An embedding vector."""
    vector: List[float]
    text: str
    model: str


@dataclass
class AudioTranscription:
    """A transcription of audio to text."""
    text: str
    language: Optional[str] = None
    confidence: Optional[float] = None


@dataclass
class AudioSynthesis:
    """A synthesis of text to audio."""
    audio: bytes
    format: str
    model: str


class LanguageModel(BaseModel, ABC):
    """Base class for language models."""

    @abstractmethod
    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model."""
        pass


class EmbeddingModel(BaseModel, ABC):
    """Base class for embedding models."""

    @abstractmethod
    async def embed(self, texts: List[str], **kwargs) -> List[List[float]]:
        """Embed texts into vectors."""
        pass


class SpeechToTextModel(BaseModel, ABC):
    """Base class for speech-to-text models."""

    @abstractmethod
    async def transcribe(self, audio_file: str, **kwargs) -> str:
        """Transcribe audio to text."""
        pass


class TextToSpeechModel(BaseModel, ABC):
    """Base class for text-to-speech models."""

    @abstractmethod
    def synthesize(
        self, text: str, voice: Optional[str] = None, **kwargs
    ) -> bytes:
        """Synthesize text to speech."""
        pass



import os
from typing import Any, Dict

from langchain_core.language_models.chat_models import BaseChatModel
from langchain_ollama.chat_models import ChatOllama

from esperanto.base.types import LanguageModel


class OllamaLanguageModel(LanguageModel):
    """Ollama language model implementation."""

    def __init__(self, model_name: str, config: Dict[str, Any] = {}):
        super().__init__(model_name, config)
        self.max_tokens = self.config.get("max_tokens", 850)
        self.temperature = self.config.get("temperature", 1.0)
        self.streaming = self.config.get("streaming", False)
        self.top_p = self.config.get("top_p", 0.9)
        self.base_url = self.config.get("base_url") or os.environ.get(
            "OLLAMA_API_BASE", "http://localhost:11434"
        )

    @property
    def provider(self) -> str:
        return "ollama"

    def validate_config(self) -> None:
        """Validate Ollama configuration."""
        if not self.model_name:
            raise ValueError("model_name must be specified for Ollama language model")
        if not self.base_url or self.base_url.strip() == "":
            raise ValueError(
                "base_url must be specified in config or OLLAMA_API_BASE environment variable"
            )

    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model."""
        return ChatOllama(
            model=self.model_name,
            temperature=self.temperature,
            stream=self.streaming,  # ChatOllama uses stream instead of streaming
            base_url=self.base_url,
        )



"""Vertex AI embedding model implementation."""

from typing import Any, Dict, List, Optional

from vertexai.language_models import TextEmbeddingInput, TextEmbeddingModel

from esperanto.base.types import EmbeddingModel


class VertexEmbeddingModel(EmbeddingModel):
    """Vertex AI embedding model implementation."""

    def __init__(self, model_name: str, config: Optional[Dict[str, Any]] = None):
        if not model_name:
            raise ValueError("model_name must be specified for Vertex AI embedding model")
        super().__init__(model_name, config)
        self._model = None

    @property
    def model(self) -> TextEmbeddingModel:
        """Lazy initialization of Vertex AI model."""
        if self._model is None:
            self._model = TextEmbeddingModel.from_pretrained(self.model_name)
        return self._model

    @property
    def provider(self) -> str:
        return "vertex"

    def validate_config(self) -> None:
        """Validate Vertex AI configuration."""
        pass

    async def embed(self, texts: List[str], **kwargs) -> List[List[float]]:
        """Generate embeddings for the given texts."""
        inputs = [TextEmbeddingInput(text) for text in texts]
        embeddings = self.model.get_embeddings(inputs)
        return [embedding.values for embedding in embeddings]



"""Ollama embedding model implementation."""

import os
from typing import Any, Dict, List, Optional

import requests

from esperanto.base.types import EmbeddingModel


class OllamaEmbeddingModel(EmbeddingModel):
    """Ollama embedding model implementation."""

    def __init__(self, model_name: str, config: Optional[Dict[str, Any]] = None):
        if not model_name:
            raise ValueError("model_name must be specified for Ollama embedding model")
        super().__init__(model_name, config)
        
        # Get base_url from config or environment
        base_url = None
        if self.config and "base_url" in self.config:
            base_url = self.config["base_url"]
        if not base_url:
            base_url = os.environ.get("OLLAMA_API_BASE")
        if not base_url:
            raise ValueError(
                "base_url must be specified in config or OLLAMA_API_BASE environment variable"
            )
        self.base_url = base_url

    @property
    def provider(self) -> str:
        return "ollama"

    def validate_config(self) -> None:
        """Validate Ollama configuration."""
        pass

    async def embed(self, texts: List[str], **kwargs) -> List[List[float]]:
        """Generate embeddings for the given texts."""
        results = []
        for text in texts:
            text = text.replace("\n", " ")
            response = requests.post(
                f"{self.base_url}/api/embed",
                json={"model": self.model_name, "input": [text]},
                **kwargs,
            )
            response_json = response.json()
            if "embeddings" not in response_json or not response_json["embeddings"]:
                raise ValueError("Invalid response from Ollama API")
            results.append(response_json["embeddings"][0])
        return results



from typing import Any, Dict, Optional

from groq import AsyncGroq

from esperanto.base.types import SpeechToTextModel


class GroqSpeechToTextModel(SpeechToTextModel):
    """Groq speech-to-text model implementation."""

    def __init__(
        self,
        model_name: str,
        config: Optional[Dict[str, Any]] = None,
    ):
        super().__init__(model_name, config)
        self._client = None
        self.language = self.config.get("language")
        self.prompt = self.config.get("prompt")
        self.response_format = self.config.get("response_format", "text")
        self.temperature = self.config.get("temperature", 0)
        self.validate_config()

    @property
    def client(self) -> AsyncGroq:
        """Get the Groq client."""
        if self._client is None:
            self._client = AsyncGroq()
        return self._client

    @client.setter
    def client(self, value: AsyncGroq) -> None:
        """Set the Groq client."""
        self._client = value

    @client.deleter
    def client(self) -> None:
        """Delete the Groq client."""
        self._client = None

    @property
    def provider(self) -> str:
        return "groq"

    def validate_config(self) -> None:
        """Validate Groq configuration."""
        if not self.model_name:
            raise ValueError(
                "model_name must be specified for Groq speech-to-text model"
            )
        if self.response_format not in ["json", "text"]:
            raise ValueError("response_format must be one of: json, text")

    async def transcribe(self, audio_file: str, **kwargs) -> str:
        """
        Transcribe audio file to text using Groq's speech-to-text API.

        Args:
            audio_file: Path to the audio file to transcribe.

        Returns:
            str: The transcribed text.

        Raises:
            ValueError: If the audio file cannot be read or transcription fails.
        """
        try:
            with open(audio_file, "rb") as f:
                response = await self.client.audio.transcriptions.create(
                    model=self.model_name,
                    file=f,
                    language=self.language,
                    prompt=self.prompt,
                    response_format=self.response_format,
                    temperature=self.temperature,
                    **kwargs,
                )

                return response

        except Exception as e:
            raise ValueError(f"Failed to transcribe audio: {str(e)}")



import os
from typing import Any, Dict

from langchain_core.language_models.chat_models import BaseChatModel
from langchain_openai import ChatOpenAI
from pydantic import SecretStr

from esperanto.base.types import LanguageModel


class OpenAILanguageModel(LanguageModel):
    """OpenAI language model implementation."""

    def __init__(
        self,
        model_name: str,
        config: Dict[str, Any] = {},
    ):
        if not model_name:
            raise ValueError("model_name must be specified for OpenAI language model")
        super().__init__(model_name, config)
        self.max_tokens: int = self.config.get("max_tokens", 850)
        self.temperature: float = self.config.get("temperature", 1.0)
        self.streaming: bool = self.config.get("streaming", True)
        self.top_p: float = self.config.get("top_p", 0.9)
        self.json_mode: bool = self.config.get("json", False)

        # Handle API configuration
        api_key = self.config.get("api_key") or os.environ.get(
            "OPENAI_API_KEY", ""
        )
        self.api_key: SecretStr = SecretStr(str(api_key)) if api_key else SecretStr("")
        self.base_url: str | None = self.config.get("openai_api_base") or os.environ.get(
            "OPENAI_API_BASE", None
        )
        self.organization: str | None = self.config.get("organization") or os.environ.get(
            "OPENAI_ORGANIZATION", None
        )

    @property
    def provider(self) -> str:
        return "openai"

    def validate_config(self) -> None:
        """Validate OpenAI configuration."""
        if not self.model_name:
            raise ValueError("model_name must be specified for OpenAI language model")
        if not self.api_key.get_secret_value():
            raise ValueError(
                "api_key must be specified in config or OPENAI_API_KEY environment variable"
            )

    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model."""
        model_kwargs: Dict[str, Any] = {
            "response_format": {"type": "json"} if self.json_mode else None
        }

        return ChatOpenAI(
            model=self.model_name,
            max_tokens=self.max_tokens,
            temperature=self.temperature,
            streaming=self.streaming,
            top_p=self.top_p,
            api_key=self.api_key,
            base_url=self.base_url,
            organization=self.organization,
            model_kwargs=model_kwargs,
        )



"""Base package for esperanto."""

from .model import BaseModel
from .types import (
    AudioSynthesis,
    AudioTranscription,
    ChatCompletion,
    Embedding,
    EmbeddingModel,
    LanguageModel,
    Message,
    SpeechToTextModel,
    TextToSpeechModel,
)

__all__ = [
    "BaseModel",
    "AudioSynthesis",
    "AudioTranscription",
    "ChatCompletion",
    "Embedding",
    "EmbeddingModel",
    "LanguageModel",
    "Message",
    "SpeechToTextModel",
    "TextToSpeechModel",
]



"""Google Cloud text-to-speech model implementation."""

from typing import Any, Dict, Optional

from google.cloud import texttospeech_v1

from esperanto.base.types import TextToSpeechModel


class GeminiTextToSpeechModel(TextToSpeechModel):
    """Google Cloud text-to-speech model implementation."""

    def __init__(
        self,
        model_name: str = "en-US-Standard-A",
        config: Optional[Dict[str, Any]] = None,
    ):
        super().__init__(model_name, config)
        self._client = None
        self.model = model_name
        self.language_code = self.config.get("language_code", "en-US")
        self.voice_name = self.config.get("voice_name", "en-US-Standard-A")
        self.speaking_rate = self.config.get("speaking_rate", 1.0)
        self.pitch = self.config.get("pitch", 0.0)
        self.volume_gain_db = self.config.get("volume_gain_db", 0.0)
        self.audio_encoding = self.config.get("audio_encoding", "MP3")
        self.sample_rate_hertz = self.config.get("sample_rate_hertz", 16000)

    @property
    def client(self) -> texttospeech_v1.TextToSpeechAsyncClient:
        """Get the Google Cloud Text-to-Speech client."""
        if self._client is None:
            self._client = texttospeech_v1.TextToSpeechAsyncClient()
        return self._client

    @client.setter
    def client(self, value: texttospeech_v1.TextToSpeechAsyncClient) -> None:
        """Set the Google Cloud Text-to-Speech client."""
        self._client = value

    @client.deleter
    def client(self) -> None:
        """Delete the Google Cloud Text-to-Speech client."""
        self._client = None

    @property
    def provider(self) -> str:
        """Get provider name."""
        return "gemini"

    def validate_config(self) -> None:
        """Validate Google Cloud Text-to-Speech configuration."""
        if not self.model:
            raise ValueError("model must be specified for Google Cloud text-to-speech model")
        if not self.language_code:
            raise ValueError("language_code must be specified")
        if not self.voice_name:
            raise ValueError("voice_name must be specified")
        if not 0.25 <= self.speaking_rate <= 4.0:
            raise ValueError("speaking_rate must be between 0.25 and 4.0")
        if not -20.0 <= self.pitch <= 20.0:
            raise ValueError("pitch must be between -20.0 and 20.0")
        if not -96.0 <= self.volume_gain_db <= 16.0:
            raise ValueError("volume_gain_db must be between -96.0 and 16.0")
        if not 8000 <= self.sample_rate_hertz <= 48000:
            raise ValueError("sample_rate_hertz must be between 8000 and 48000")

    async def synthesize(self, text: str, output_file: str, **kwargs: Any) -> str:
        """Synthesize text to speech and save to file.
        
        Args:
            text: Text to synthesize
            output_file: Path to save the audio file
            **kwargs: Additional arguments for synthesis
            
        Returns:
            Path to the output audio file
        """
        self.validate_config()

        # Create input
        synthesis_input = texttospeech_v1.SynthesisInput(text=text)

        # Create voice config
        voice = texttospeech_v1.VoiceSelectionParams(
            language_code=self.language_code,
            name=self.voice_name,
        )

        # Create audio config
        audio_config = texttospeech_v1.AudioConfig(
            audio_encoding=getattr(texttospeech_v1.AudioEncoding, self.audio_encoding),
            speaking_rate=kwargs.get("speaking_rate", self.speaking_rate),
            pitch=kwargs.get("pitch", self.pitch),
            volume_gain_db=kwargs.get("volume_gain_db", self.volume_gain_db),
            sample_rate_hertz=kwargs.get("sample_rate_hertz", self.sample_rate_hertz),
        )

        # Generate audio
        response = await self.client.synthesize_speech(
            input=synthesis_input,
            voice=voice,
            audio_config=audio_config,
        )

        # Save to file
        with open(output_file, "wb") as f:
            f.write(response.audio_content)

        return output_file



"""Gemini language model implementation."""

import os
from typing import Any, Dict

from langchain_google_genai import ChatGoogleGenerativeAI
from pydantic import SecretStr

from esperanto.base.types import LanguageModel


class GeminiLanguageModel(LanguageModel):
    """Gemini language model implementation."""

    def __init__(
        self,
        model_name: str,
        config: Dict[str, Any] = {},
    ) -> None:
        """Initialize Gemini language model.

        Args:
            model_name: Name of the model to use.
            config: Configuration for the model.

        Raises:
            ValueError: If the model name is invalid.
        """
        if not model_name or len(model_name.strip()) == 0:
            raise ValueError("model_name must be specified for Gemini language model")

        super().__init__(model_name=model_name, config=config or {})
        self.streaming = True  # Only use streaming for consistency
        self.temperature = config.get("temperature", 0.7)
        self.top_p = config.get("top_p", 0.9)
        self.max_tokens = config.get("max_tokens", 850)

        # Handle API key
        api_key = config.get("api_key") or os.environ.get("GEMINI_API_KEY", "")
        self._api_key = api_key

    @property
    def provider(self) -> str:
        """Get the provider name.

        Returns:
            Provider name.
        """
        return "gemini"

    def validate_config(self) -> None:
        """Validate the model configuration.

        Raises:
            ValueError: If the configuration is invalid.
        """
        if not self.model_name or len(self.model_name.strip()) == 0:
            raise ValueError("model_name must be specified for Gemini language model")

        if self.temperature is not None and not 0 <= self.temperature <= 1:
            raise ValueError("temperature must be between 0 and 1")

        if self.top_p is not None and not 0 <= self.top_p <= 1:
            raise ValueError("top_p must be between 0 and 1")

        if self.max_tokens is not None and not isinstance(self.max_tokens, int):
            raise ValueError("max_tokens must be an integer")

    def to_langchain(self) -> ChatGoogleGenerativeAI:
        """Convert to LangChain model.

        Returns:
            LangChain model instance.
        """
        self.validate_config()

        model = ChatGoogleGenerativeAI(
            model=f"models/{self.model_name}",
            api_key=SecretStr(self._api_key),
            temperature=self.temperature,
            top_p=self.top_p,
            max_tokens=self.max_tokens,
        )

        return model



"""Speech-to-text model implementations."""

from esperanto.providers.speech_to_text.groq import GroqSpeechToTextModel
from esperanto.providers.speech_to_text.openai import OpenAISpeechToTextModel

__all__ = [
    "OpenAISpeechToTextModel",
    "GroqSpeechToTextModel",
]



import os
from typing import Any, Dict

from langchain_core.language_models.chat_models import BaseChatModel
from langchain_groq import ChatGroq

from esperanto.base.types import LanguageModel


class GroqLanguageModel(LanguageModel):
    """Groq language model implementation."""

    def __init__(
        self,
        model_name: str,
        config: Dict[str, Any] = {},
    ):
        if not model_name:
            raise ValueError("model_name must be specified for Groq language model")
        super().__init__(model_name, config)
        self.api_key = self.config.get("api_key", os.environ.get("GROQ_API_KEY"))
        self.max_tokens = self.config.get("max_tokens", 850)
        self.temperature = self.config.get("temperature", 1.0)
        self.streaming = self.config.get("streaming", True)
        self.top_p = self.config.get("top_p", 0.9)

    @property
    def provider(self) -> str:
        return "groq"

    def validate_config(self) -> None:
        """Validate Groq configuration."""
        if not self.model_name:
            raise ValueError("model_name must be specified for Groq language model")

    def to_langchain(self) -> BaseChatModel:
        """Convert to a LangChain chat model."""
        return ChatGroq(
            api_key=self.api_key,
            model=self.model_name,
            max_tokens=self.max_tokens,
            temperature=self.temperature,
            streaming=self.streaming,
            model_kwargs={"top_p": self.top_p},
        )
